package com.happy.dao;

import com.happy.Model.Advertise;

import java.util.List;

public interface AdvertiseDao {

    public int insertAdvertise(Advertise advertise);

    public int updateAdvertise(Advertise advertise);

    public int delAdvertise(int id);

    // 查询已开启的广告
    public List<Advertise> queryOpenAdvertise();

    public Advertise queryById(int id);

    public Advertise queryByTitle(String title);

    public List<Advertise> queryAdvertisePage(int page, int rows, String sqlx);

    public int queryAdvertiseTotal(String sqlx);

    public int updateState(Integer id, Integer state,String create_time);

}
